@@ -3,8 +3,7 @@
  * \file     editDlg.c
  * \brief    \e Edit dialog(s).
  *
- * \author   Copyright (C) 2006, 2011, 2012 Ralf Hoppe <ralf.hoppe@ieee.org>
- * \version  $Id: editDlg.c 240 2012-06-08 15:52:14Z ralf $
+ * \author   Copyright (C) 2006, 2011, 2012, 2013, 2020 Ralf Hoppe <ralf.hoppe@dfcgen.de>
  *
  ******************************************************************************/
 
@@ -282,8 +281,7 @@ static GtkResponseType editDlgSettingsAccept (GtkWidget* dialog)
         settings.frequUnit = unitF[idxFrequ];
 
         cfgSetDesktopPrefs (&settings);               /* save new preferences */
-        responseWinRedraw (RESPONSE_TYPE_SIZE);           /* redraw all plots */
-        rootsPlotRedraw ();
+        mainDlgRedrawAll ();            /* redraw all plots and coefficients */
 
         return GTK_RESPONSE_OK;
     } /* if */
@@ -316,7 +314,7 @@ static GtkWidget* createInfoDlg (const DFCPRJ_INFO *pInfo)
     if (pixbuf != NULL)
     {
         gtk_window_set_icon (GTK_WINDOW (dialog), pixbuf);
-        gdk_pixbuf_unref (pixbuf);
+        g_object_unref (pixbuf);
     } /* if */
 
 
