@@ -3,8 +3,7 @@
  * \file
  *           Linear FIR filter dialog functions.
  *
- * \author   Copyright (C) 2006, 2011-2012 Ralf Hoppe
- * \version  $Id: linFirDesignDlg.c 179 2012-05-19 15:56:13Z ralf $
+ * \author   Copyright (C) 2006, 2011-2012, 2020 Ralf Hoppe
  *
  ******************************************************************************/
 
@@ -83,12 +82,12 @@ typedef struct
 /** Array of all standard FIR filter types (in same order as the enums).
  */
 static LINFIRDLG_CHAR linFirDlgChar[LINFIR_TYPE_SIZE] =
-{   /* type,            btn,  name */
-    {LINFIR_TYPE_RECT,  NULL, N_("Rectangular (perfect)")},
-    {LINFIR_TYPE_COS,   NULL, N_("Cosine")},
-    {LINFIR_TYPE_COS2,  NULL, N_("Squared cosine")},
-    {LINFIR_TYPE_GAUSS, NULL, N_("Gaussian")},
-    {LINFIR_TYPE_SQR,   NULL, N_("Squared 1st order")}
+{   /* type, btn, name */
+    [LINFIR_TYPE_RECT] = {LINFIR_TYPE_RECT,  NULL, N_("Rectangular (perfect)")},
+    [LINFIR_TYPE_COS] = {LINFIR_TYPE_COS,   NULL, N_("Cosine")},
+    [LINFIR_TYPE_COS2] = {LINFIR_TYPE_COS2,  NULL, N_("Squared cosine")},
+    [LINFIR_TYPE_GAUSS] = {LINFIR_TYPE_GAUSS, NULL, N_("Gaussian")},
+    [LINFIR_TYPE_SQR] = {LINFIR_TYPE_SQR,   NULL, N_("Squared 1st order")}
 }; /* linFirDlgChar[] */
 
 
@@ -96,20 +95,20 @@ static LINFIRDLG_CHAR linFirDlgChar[LINFIR_TYPE_SIZE] =
  */
 static LINFIRDLG_DSPWIN linFirDlgWin[LINFIR_DSPWIN_SIZE] =
 { /* type, btn, name */
-    {LINFIR_DSPWIN_RECT, NULL, N_("None")},             /* LINFIR_DSPWIN_RECT */
-    {LINFIR_DSPWIN_HAMMING, NULL, N_("Hamming")},    /* LINFIR_DSPWIN_HAMMING */
-    {LINFIR_DSPWIN_HANNING, NULL, N_("Hanning")},    /* LINFIR_DSPWIN_HANNING */
-    {LINFIR_DSPWIN_BLACKMAN, NULL, N_("Blackman")}, /* LINFIR_DSPWIN_BLACKMAN */
-    {LINFIR_DSPWIN_KAISER, NULL, N_("Kaiser")},       /* LINFIR_DSPWIN_KAISER */
+    [LINFIR_DSPWIN_RECT] = {LINFIR_DSPWIN_RECT, NULL, N_("None")},
+    [LINFIR_DSPWIN_HAMMING] = {LINFIR_DSPWIN_HAMMING, NULL, N_("Hamming")},
+    [LINFIR_DSPWIN_VANHANN] = {LINFIR_DSPWIN_VANHANN, NULL, N_("van Hann")},
+    [LINFIR_DSPWIN_BLACKMAN] = {LINFIR_DSPWIN_BLACKMAN, NULL, N_("Blackman")},
+    [LINFIR_DSPWIN_KAISER] = {LINFIR_DSPWIN_KAISER, NULL, N_("Kaiser")}
 };
 
 
-static const gboolean ftrEntry[FTR_SIZE][4] =
-{ /* entryCutF, entryCenterF, entryBandwidth, checkGeometric */
-    {TRUE,      FALSE,        FALSE,          FALSE},          /* FTR_NON */
-    {FALSE,     TRUE,         FALSE,          FALSE},     /* FTR_HIGHPASS */
-    {FALSE,     TRUE,         TRUE,           TRUE},      /* FTR_BANDPASS */
-    {FALSE,     TRUE,         TRUE,           TRUE},      /* FTR_BANDSTOP */
+static const gboolean ftrEntry[FTR_SIZE][5] =
+{                  /* entryCutF, entryCenterF, entryBandwidth, checkGeometric, evenDegree */
+    [FTR_NON]      = {TRUE,      FALSE,        FALSE,          FALSE,          FALSE},
+    [FTR_HIGHPASS] = {FALSE,     TRUE,         FALSE,          FALSE,          TRUE},
+    [FTR_BANDPASS] = {FALSE,     TRUE,         TRUE,           TRUE,           TRUE},
+    [FTR_BANDSTOP] = {FALSE,     TRUE,         TRUE,           TRUE,           TRUE},
 };
 
 
@@ -681,8 +680,7 @@ BOOL linFirDesignDlgActive (GtkWidget *topWidget)
  *                        filter must be seen as invalid. You can use the
  *                        FLTERR_CRITICAL macro from filterSupport.h to check
  *                        this condition.
- *  \todo               Share code with IIR design dialog.
- ******************************************************************************/
+  ******************************************************************************/
 int linFirDesignDlgApply (GtkWidget *topWidget, const CFG_DESKTOP* pPrefs)
 {
     GtkWidget *combo;
@@ -726,6 +724,11 @@ int linFirDesignDlgApply (GtkWidget *topWidget, const CFG_DESKTOP* pPrefs)
             design.ftr.type = idx;
         } /* else */
 
+        if (ftrEntry[design.ftr.type][4] && GSL_IS_ODD(design.order))
+        {
+            dlgError(topWidget, _("Degree must be even for frequency transformation."));
+            return INT_MAX;
+        }
 
         if (ftrEntry[design.ftr.type][0]) /* lowpass cutoff frequency required? */
         {
