@@ -3,8 +3,7 @@
  * \file
  *           Linear FIR filter coefficients generator.
  *
- * \author   Copyright (C) 2006, 2011, 2012 Ralf Hoppe
- * \version  $Id: linFirFilter.c 202 2012-05-29 20:07:37Z ralf $
+ * \author   Copyright (C) 2006, 2011, 2012, 2020 Ralf Hoppe
  *
  ******************************************************************************/
 
@@ -40,17 +39,30 @@
 typedef int (*LINFIR_SYSGEN_FUNC)(double x, MATHPOLY *poly);
 
 
-/* FUNCTION *******************************************************************/
-/** Mathematical function \f$y=f(x;a)\f$ with parameter \f$a\f$.
+/**
+ *  \brief Additive inversion (LP/HP frequency transformation)
+ *         cutoff frequency correction for linear FIR systems.
  *
- *  \param x            Argument \f$x\f$ between 0 and 1.
- *  \param parameter    Parameter \f$a\f$.
+ *  \param fc           Cutoff frequency.
+ *
+ *  \return             Corrected cutoff frequency.
+ ******************************************************************************/
+typedef double (*LINFIR_FCORR_FUNC)(double x);
+
+
+/* FUNCTION *****************************************************************/
+/**
+ *  \brief Mathematical function \f$y=f(x;a)\f$ with parameter \f$a\f$.
+ *
+ *  \param step         windowing step, running from 0 to \p degree
+ *  \param degree       degree of filter
+ *  \param param        additional window parameter \f$a\f$
  *
  *  \return             Result \f$y\f$ when successful evaluated, else
  *                      GSL_POSINF or GSL_NEGINF. Use the functions gsl_isinf()
  *                      or gsl_finite() for result checking.
- ******************************************************************************/
-typedef double (*LINFIR_WINDOW_FUNC)(double x, double parameter);
+ ****************************************************************************/
+typedef double (*LINFIR_WINDOW_FUNC)(int step, int degree, double param);
 
 
 
@@ -84,11 +96,16 @@ static int genGaussianSystem (double x, MATHPOLY *poly);
 static int genCosineSystem (double x, MATHPOLY *poly);
 static int genCosine2System (double x, MATHPOLY *poly);
 static int genSquaredSystem (double x, MATHPOLY *poly);
-static double firWinKaiser (double x, double parameter);
-static double firWinRectangle (double x, double parameter);
-static double firWinHamming (double x, double parameter);
-static double firWinHanning (double x, double parameter);
-static double firWinBlackman (double x, double parameter);
+static double corrRectangularCutoff (double fc);
+static double corrGaussianCutoff (double fc);
+static double corrCosineCutoff (double fc);
+static double corrCosine2Cutoff (double fc);
+static double corrSquaredCutoff (double fc);
+static double firWinKaiser (int step, int degree, double param);
+static double firWinRectangle (int step, int degree, double param);
+static double firWinHamming (int step, int degree, double param);
+static double firWinVanHann (int step, int degree, double param);
+static double firWinBlackman (int step, int degree, double param);
 static double ftrHighpass (FLTCOEFF *pFilter);
 static double ftrBandpass (FLTCOEFF *pFilter, double fc, double bw, BOOL geometric);
 
@@ -102,9 +119,10 @@ static double ftrBandpass (FLTCOEFF *pFilter, double fc, double bw, BOOL geometr
 /** Rectangular magnitude response system generator. Calculation of coefficient
  *  \f$c_i\f$ is based on the argument \f$x\f$ and polynomial degree \f$n\f$
  *  following the formula:
-    \f[
-        c_i=\frac{\sin\left[2\pi x\,(i-n/2)\right]}{2\pi x\,(i-n/2)}
-    \f]
+    \f{eqnarray*}
+        c_i &=& \frac{\sin\left[2\pi x\,(i-n/2)\right]}{2\pi x\,(i-n/2)} \\
+            &=& \si\left[2\pi x\,(i-n/2)\right]
+    \f}
  *
  *  \param x            Argument \f$x\f$.
  *  \param poly         The numerator polynomial of the linear FIR system (in
@@ -180,9 +198,11 @@ static int genCosineSystem (double x, MATHPOLY *poly)
 /** Squared cosine magnitude response system generator. Calculation of
  *  coefficient \f$c_i\f$ is based on the argument \f$x\f$ and polynomial degree
  *  \f$n\f$ following the formula:
-    \f[
-        c_i=\frac{}{}
-    \f]
+    \f{eqnarray*}
+        \Lambda &=& \frac{\pi}{\arccos\frac{1}{\sqrt[4]{2}}} \\
+        c_i &=& \frac{\si\left[\Lambda \pi x\,(i-n/2)\right]}{1-\left[\Lambda x\,(i-n/2)\right]^2} \\
+        \si(x) &=& \frac{\sin x}{x}
+    \f}
  *
  *  \param x            Argument \f$x\f$.
  *  \param poly         The numerator polynomial of the linear FIR system (in
@@ -190,7 +210,6 @@ static int genCosineSystem (double x, MATHPOLY *poly)
  *
  *  \return             Zero on success, else an error code (from errno.h or
  *                      gsl_errno.h).
- *  \todo               Give formula
  ******************************************************************************/
 static int genCosine2System (double x, MATHPOLY *poly)
 {
@@ -246,11 +265,11 @@ static int genSquaredSystem (double x, MATHPOLY *poly)
     gsl_sf_result result;
 
     double deg2 = poly->degree / 2.0;
-    double constant = 2.0 * M_PI / sqrt (M_SQRT2 - 1.0);
+    double constant = -2.0 * M_PI / sqrt (M_SQRT2 - 1.0);
 
     for (i = 0; i <= poly->degree; i++)
     {
-        err = gsl_sf_exp_e (-constant * x * fabs ((i - deg2)), &result);
+        err = gsl_sf_exp_e (constant * x * fabs (deg2 - i), &result);
 
         if (err != GSL_SUCCESS)
         {
@@ -305,84 +324,161 @@ static int genGaussianSystem (double x, MATHPOLY *poly)
 } /* genGaussianSystem() */
 
 
+/**
+ *  \brief      Cutoff frequency correction for LP/HP transformation of a
+ *              rectangular system.
+ *
+ *  \param[in]  fc      target cutoff frequency of highpass
+ *
+ *  \return     corrected cutoff frequency, to be used for lowpass design
+ */
+static double corrRectangularCutoff (double fc)
+{
+    return fc;                       /* no correction on rectangular system */
+}
+
+
+/**
+ *  \brief      Cutoff frequency correction for LP/HP transformation of a
+ *              cosine system.
+ *
+ *  \param[in]  fc      target cutoff frequency of highpass
+ *
+ *  \return     corrected cutoff frequency, to be used for lowpass design
+ */
+static double corrCosineCutoff (double fc)
+{
+    return M_PI_4 * fc / acos (1.0 - M_SQRT1_2);
+}
+
+
+/**
+ *  \brief      Cutoff frequency correction for LP/HP transformation of a
+ *              squared cosine system.
+ *
+ *  \param[in]  fc      target cutoff frequency of highpass
+ *
+ *  \return     corrected cutoff frequency, to be used for lowpass design
+ */
+static double corrCosine2Cutoff (double fc)
+{
+    return fc * acos (sqrt (M_SQRT1_2)) / acos (sqrt (1.0 - M_SQRT1_2));
+}
+
+
+/**
+ *  \brief      Cutoff frequency correction for LP/HP transformation of a
+ *              1st order squared system.
+ *
+ *  \param[in]  fc      target cutoff frequency of highpass
+ *
+ *  \return     corrected cutoff frequency, to be used for lowpass design
+ */
+static double corrSquaredCutoff (double fc)
+{
+    return (M_SQRT2 - 1.0) * fc;
+}
+
+
+/**
+ *  \brief      Cutoff frequency correction for LP/HP transformation of a
+ *              gaussian system.
+ *
+ *  \param[in]  fc      target cutoff frequency of highpass
+ *
+ *  \return     corrected cutoff frequency, to be used for lowpass design
+ */
+static double corrGaussianCutoff (double fc)
+{
+    return M_SQRT1_2 * fc * sqrt (- M_LN2 / log (1.0 - M_SQRT1_2));
+}
+
 
 /* FUNCTION *******************************************************************/
 /** Rectangular window function.
  *
- *  \param x            Argument \f$x\f$ between 0 and 1.
- *  \param parameter    Parameter (unused).
+ *  \param step         windowing step, running from 0 to \p degree
+ *  \param degree       degree of filter
+ *  \param param        parameter (unused)
  *
  *  \return             Result \f$y\f$ when successful evaluated, else
  *                      GSL_POSINF or GSL_NEGINF. Use the functions gsl_isinf()
  *                      or gsl_finite() for result checking.
  ******************************************************************************/
-static double firWinRectangle (double x, double parameter)
+static double firWinRectangle (int step, int degree, double param)
 {
-    return mathFuncRectangle (x);
+    (void) param;
+    return mathFuncRectangle ((double) step / degree);
 } /* firWinRectangle() */
 
 
 /* FUNCTION *******************************************************************/
 /** \e Hamming window function.
  *
- *  \param x            Argument \f$x\f$ between 0 and 1.
- *  \param parameter    Parameter (unused).
+ *  \param step         windowing step, running from 0 to \p degree
+ *  \param degree       degree of filter
+ *  \param param        parameter (unused)
  *
  *  \return             Result \f$y\f$ when successful evaluated, else
  *                      GSL_POSINF or GSL_NEGINF. Use the functions gsl_isinf()
  *                      or gsl_finite() for result checking.
  ******************************************************************************/
-static double firWinHamming (double x, double parameter)
+static double firWinHamming (int step, int degree, double param)
 {
-    return mathFuncHamming (x);
+    (void) param;
+    return mathFuncHamming ((double) step / degree);
 } /* firWinHamming() */
 
 
 /* FUNCTION *******************************************************************/
-/** \e Hanning window function.
+/** \e van \e Hann window function.
  *
- *  \param x            Argument \f$x\f$ between 0 and 1.
- *  \param parameter    Parameter (unused).
+ *  \param step         windowing step, running from 0 to \p degree
+ *  \param degree       degree of filter
+ *  \param param        parameter (unused)
  *
  *  \return             Result \f$y\f$ when successful evaluated, else
  *                      GSL_POSINF or GSL_NEGINF. Use the functions gsl_isinf()
  *                      or gsl_finite() for result checking.
  ******************************************************************************/
-static double firWinHanning (double x, double parameter)
+static double firWinVanHann (int step, int degree, double param)
 {
-    return mathFuncHanning (x);
-} /* firWinHanning() */
+    (void) param;
+    return mathFuncVanHann ((double) (step + 1) / (degree + 2));
+} /* firWinVanHann() */
 
 
 /* FUNCTION *******************************************************************/
 /** \e Blackman window function.
  *
- *  \param x            Argument \f$x\f$ between 0 and 1.
- *  \param parameter    Parameter (unused).
+ *  \param step         windowing step, running from 0 to \p degree
+ *  \param degree       degree of filter
+ *  \param param        parameter (unused)
  *
  *  \return             Result \f$y\f$ when successful evaluated, else
  *                      GSL_POSINF or GSL_NEGINF. Use the functions gsl_isinf()
  *                      or gsl_finite() for result checking.
  ******************************************************************************/
-static double firWinBlackman (double x, double parameter)
+static double firWinBlackman (int step, int degree, double param)
 {
-    return mathFuncBlackman (x);
+    return mathFuncBlackman ((double) (step + 1) / (degree + 2));
 } /* firWinBlackman() */
 
 
 /* FUNCTION *******************************************************************/
-/** \e Blackman window function.
+/** \e Kaiser window function.
  *
- *  \param x            Argument \f$x\f$ between 0 and 1.
- *  \param parameter    Parameter \f$\alpha\f$ of \e Kaiser window.
+ *  \param step         windowing step, running from 0 to \p degree
+ *  \param degree       degree of filter
+ *  \param param        parameter \f$\alpha\f$ of \e Kaiser window
  *
  *  \return             Result \f$y\f$ when successful evaluated, else
  *                      GSL_POSINF or GSL_NEGINF. Use the functions gsl_isinf()
  *                      or gsl_finite() for result checking.
  ******************************************************************************/
-static double firWinKaiser (double x, double parameter)
+static double firWinKaiser (int step, int degree, double param)
 {
-    return mathFuncKaiser (x, parameter);
+    return mathFuncKaiser ((double) step / degree, param);
 } /* firWinKaiser() */
 
 
@@ -397,22 +493,22 @@ static double firWinKaiser (double x, double parameter)
         |H_{HP}(f)| = H(0) - |H_{LP}(f)|
     \f]
  *
+ *  \attention          \p pFilter->num.degree must be even (LP/HP transform
+ *                      for systems with odd order is not possible/supported).
+ *
  *  \param pFilter      Pointer to linear FIR lowpass that coefficients shall
  *                      transformed.
  *
  *  \return             The frequency where the magnitude response has it's
  *                      maximum.
- * \todo Handle odd degree
- ******************************************************************************/
+  ******************************************************************************/
 static double ftrHighpass (FLTCOEFF *pFilter)
 {
     MATHPOLY *poly = &pFilter->num;
     double magnitude = gsl_poly_eval (poly->coeff, poly->degree + 1, 1.0);
 
-    if (GSL_IS_EVEN (poly->degree))
-    {
-        poly->coeff[poly->degree / 2] -= magnitude;
-    } /* if */
+    ASSERT(GSL_IS_EVEN(poly->degree));
+    poly->coeff[poly->degree / 2] -= magnitude;
 
     return pFilter->f0 / 2.0 - DBL_EPSILON;
 } /* ftrHighpass() */
@@ -436,6 +532,8 @@ static double ftrHighpass (FLTCOEFF *pFilter)
     \f]
  *  which may produce an aliasing effect at \f$f=0\f$.
  *
+ *  \attention          \p pFilter->num.degree must be even (LP/BP transform
+ *                      for systems with odd order is not possible/supported).
  *
  *  \param pFilter      Pointer to linear FIR lowpass that coefficients shall
  *                      transformed.
@@ -453,16 +551,18 @@ static double ftrBandpass (FLTCOEFF *pFilter, double fc, double bw, BOOL geometr
     double factor, tmp;
 
     MATHPOLY *poly = &pFilter->num;
-    double deg2 = poly->degree / 2.0;                       /* half of degree */
+    double deg2 = poly->degree / 2.0;                     /* half of degree */
+
+    ASSERT(GSL_IS_EVEN(poly->degree));
 
     if (geometric)
     {
         fc = HYPOT (fc, 0.5 * bw);
     } /* if */
 
-    factor = 2 * M_PI * fc / pFilter->f0;
+    factor = 2.0 * M_PI * fc / pFilter->f0;
 
-    for (i = 0; i <= poly->degree / 2; i++)             /* modify coefficients */
+    for (i = 0; i <= poly->degree / 2; i++)          /* modify coefficients */
     {
         tmp = cos (factor * (i - deg2));
         poly->coeff[i] *= tmp;
@@ -476,6 +576,9 @@ static double ftrBandpass (FLTCOEFF *pFilter, double fc, double bw, BOOL geometr
 
 /* FUNCTION *******************************************************************/
 /** \e Z domain bandstop transformation.
+ *
+ *  \attention          \p pFilter->num.degree must be even (LP/BS transform
+ *                      for systems with odd order is not possible/supported).
  *
  *  \param pFilter      Pointer to linear FIR lowpass that coefficients shall
  *                      transformed.
@@ -494,13 +597,11 @@ static double ftrBandstop (FLTCOEFF *pFilter, double fc, double bw, BOOL geometr
 
     MATHPOLY *poly = &pFilter->num;
 
-    fc = ftrBandpass (pFilter, fc, bw, geometric);
-    magnitude = filterResponsePoly (2 * M_PI * fc / pFilter->f0, poly);
+    ASSERT(GSL_IS_EVEN(poly->degree));
 
-    if (GSL_IS_EVEN (poly->degree))
-    {
-        poly->coeff[poly->degree / 2] -= magnitude;
-    } /* if */
+    fc = ftrBandpass (pFilter, fc, bw, geometric);
+    magnitude = filterResponsePoly (2.0 * M_PI * fc / pFilter->f0, poly);
+    poly->coeff[poly->degree / 2] -= magnitude;
 
     return pFilter->f0 / 2.0 - DBL_EPSILON;
 } /* ftrBandstop() */
@@ -511,8 +612,13 @@ static double ftrBandstop (FLTCOEFF *pFilter, double fc, double bw, BOOL geometr
 
 
 /* FUNCTION *******************************************************************/
-/** Generates a linear FIR filter. The cutoff frequency always is assumed to be
- *  the 3dB point of magnitude response.
+/**
+ *  \brief Generates a linear FIR filter.
+ *
+ *  \attention          For frequency transformations (HP/BP/BS) the system
+ *                      order (degree) must be even.
+ *  \note               The cutoff frequency is assumed to be the 3dB point
+ *                      of magnitude response.
  *
  *  \param pDesign      Pointer to linear FIR filter design data.
  *  \param pFilter      Pointer to buffer which gets the generated filter.
@@ -521,38 +627,43 @@ static double ftrBandstop (FLTCOEFF *pFilter, double fc, double bw, BOOL geometr
  *
  *  \return             Zero on success, else an error number (see errno.h or
  *                      gsl_errno.h for predefined codes).
- *  \todo               Implement semantic checks on bandwidth, center and cutoff
- *                      frequency wrt. sample frequency.
- *  \todo               Correct the lowpass design cutoff frequency (see Bessel
- *                      lowpass) when performing frequency transformations
- *                      (depends on characteristic too, significant on perfect
- *                      lowpass).
- *  \todo               Use fpsetround()?
  ******************************************************************************/
 int linFirFilterGen (LINFIR_DESIGN *pDesign, FLTCOEFF *pFilter)
 {
     static const LINFIR_SYSGEN_FUNC genFuncs[LINFIR_TYPE_SIZE] =
     {
-        genRectangularSystem,                             /* LINFIR_TYPE_RECT */
-        genCosineSystem,                                   /* LINFIR_TYPE_COS */
-        genCosine2System,                                 /* LINFIR_TYPE_COS2 */
-        genSquaredSystem,                                /* LINFIR_TYPE_GAUSS */
-        genGaussianSystem                                  /* LINFIR_TYPE_SQR */
+        [LINFIR_TYPE_RECT] = genRectangularSystem,
+        [LINFIR_TYPE_COS] = genCosineSystem,
+        [LINFIR_TYPE_COS2] = genCosine2System,
+        [LINFIR_TYPE_GAUSS] = genGaussianSystem,
+        [LINFIR_TYPE_SQR] = genSquaredSystem
+    };
+
+    static const LINFIR_FCORR_FUNC corrFuncs[LINFIR_TYPE_SIZE] =
+    {
+        [LINFIR_TYPE_RECT] = corrRectangularCutoff,
+        [LINFIR_TYPE_COS] = corrCosineCutoff,
+        [LINFIR_TYPE_COS2] = corrCosine2Cutoff,
+        [LINFIR_TYPE_GAUSS] = corrGaussianCutoff,
+        [LINFIR_TYPE_SQR] = corrSquaredCutoff
     };
 
     static const LINFIR_WINDOW_FUNC winFuncs[LINFIR_DSPWIN_SIZE] =
     {
-        firWinRectangle,                                /* LINFIR_DSPWIN_RECT */
-        firWinHamming,                               /* LINFIR_DSPWIN_HAMMING */
-        firWinHanning,                               /* LINFIR_DSPWIN_HANNING */
-        firWinBlackman,                             /* LINFIR_DSPWIN_BLACKMAN */
-        firWinKaiser                                  /* LINFIR_DSPWIN_KAISER */
+        [LINFIR_DSPWIN_RECT] = firWinRectangle,
+        [LINFIR_DSPWIN_HAMMING] = firWinHamming,
+        [LINFIR_DSPWIN_VANHANN] = firWinVanHann,
+        [LINFIR_DSPWIN_BLACKMAN] = firWinBlackman,
+        [LINFIR_DSPWIN_KAISER] = firWinKaiser
     };
 
 
     int i, err;
     gsl_error_handler_t *oldHandler;
-    double fnorm;                                /* unity normation frequency */
+    double fnorm;                                   /* unity norm frequency */
+
+    ASSERT(GSL_IS_EVEN(pDesign->order) ||
+           (pDesign->ftr.type == FTR_NON));
 
     pFilter->factor = 0.0;                 /* roots are invalid (unused here) */
 
@@ -566,42 +677,41 @@ int linFirFilterGen (LINFIR_DESIGN *pDesign, FLTCOEFF *pFilter)
     pFilter->den.coeff[0] = 1.0;
 
     /* All GSL errors are handled by the caller (starting from here), therefore
-     * disable the abort behaviour of the GSL library.
+     * disable the abort behavior of the GSL library.
      */
     oldHandler = gsl_set_error_handler_off ();
 
     pFilter->factor = 0.0;            /* no valid roots representation so far */
+    ASSERT (pDesign->type < LINFIR_TYPE_SIZE);
 
-    switch (pDesign->ftr.type)                    /* frequency transformation */
+    switch (pDesign->ftr.type)                  /* frequency transformation */
     {
-        case FTR_BANDPASS:
         case FTR_BANDSTOP:
+            pDesign->cutoff = corrFuncs[pDesign->type] (0.5 * pDesign->ftr.bw);
+            break;
+
+        case FTR_BANDPASS:
             pDesign->cutoff = 0.5 * pDesign->ftr.bw;
             break; /* FTR_BANDPASS, FTR_BANDSTOP */
 
-
-        case FTR_HIGHPASS :
-            pDesign->cutoff = pDesign->ftr.fc;
+        case FTR_HIGHPASS:
+            pDesign->cutoff = corrFuncs[pDesign->type] (pDesign->ftr.fc);
             break; /* FTR_HIGHPASS */
 
-
         case FTR_NON: /* LOWPASS */
             break; /* FTR_NON */
 
-
         default:
             ASSERT(0);
     } /* switch */
 
-
-    ASSERT (pDesign->type < LINFIR_TYPE_SIZE);
     err = genFuncs[pDesign->type] (pDesign->cutoff / pFilter->f0, &pFilter->num);
     LINFIR_ERROR_RET (pFilter, err, "Linear FIR filter generation has failed");
 
-    for (i = 0; i <= pFilter->num.degree; i++)       /* apply window function */
+    for (i = 0; i <= pFilter->num.degree; i++)     /* apply window function */
     {
-        fnorm = winFuncs[pDesign->dspwin] ((double)i / pFilter->num.degree,
-                                            pDesign->winparm);
+        fnorm = winFuncs[pDesign->dspwin] (i, pFilter->num.degree, pDesign->winparm);
+
         if (gsl_finite (fnorm))
         {
             pFilter->num.coeff[i] *= fnorm;
@@ -618,23 +728,19 @@ int linFirFilterGen (LINFIR_DESIGN *pDesign, FLTCOEFF *pFilter)
                                  pDesign->ftr.flags & FTRDESIGN_FLAG_CENTER_GEOMETRIC);
             break;
 
-
         case FTR_BANDPASS:
             fnorm = ftrBandpass (pFilter, pDesign->ftr.fc, pDesign->ftr.bw,
                                  pDesign->ftr.flags & FTRDESIGN_FLAG_CENTER_GEOMETRIC);
             break;
 
-
         case FTR_HIGHPASS:                /* lowpass->highpass transformation */
             fnorm = ftrHighpass (pFilter);
             break;
 
-
         case FTR_NON:                                              /* lowpass */
             fnorm = 0.0;
             break;
 
-
         default:
             ASSERT(0);
             break;
