@@ -2,9 +2,7 @@
 /**
  * \file     mainDlg.c
  * \brief    Main dialog elements (right, bottom of window) handling.
- * \author   Copyright (C) 2006, 2011, 2012 Ralf Hoppe
- *
- * \version  $Id: mainDlg.c 256 2012-06-11 17:24:58Z ralf $
+ * \author   Copyright (C) 2006, 2011-2013, 2020 Ralf Hoppe <ralf.hoppe@dfcgen.de>
  *
  * \note     Parts of code taken over from \e glade in interface.c, which itself
  *           isn't a project file.
@@ -43,7 +41,7 @@
 
 /** Function that operates on selected coefficients.
  */
-typedef BOOL (* MAINDLG_COEFF_OPERATION) (GtkWidget *top, FLTCOEFF *pFilter,
+typedef BOOL (* MAINDLG_COEFF_OPERATION) (FLTCOEFF *pFilter,
                                           MATHPOLY *poly, int index);
 
 
@@ -82,22 +80,19 @@ typedef enum
 
 static void mainDlgDestroy (GtkObject* object, gpointer user_data);
 static void mainDlgQuit (GtkWidget* widget, gpointer user_data);
-static BOOL mainDlgCoeffEdit (GtkWidget *widget, FLTCOEFF *pFilter,
-                              MATHPOLY *poly, int index);
-static BOOL mainDlgCoeffsRound (GtkWidget *widget, FLTCOEFF *pFilter,
-                                MATHPOLY *poly, int index);
-static BOOL mainDlgCoeffsFactor (GtkWidget *widget, FLTCOEFF *pFilter,
-                                 MATHPOLY *poly, int index);
+static BOOL mainDlgCoeffEdit (FLTCOEFF *pFilter, MATHPOLY *poly, int index);
+static BOOL mainDlgCoeffsRound (FLTCOEFF *pFilter, MATHPOLY *poly, int index);
+static BOOL mainDlgCoeffsFactor (FLTCOEFF *pFilter, MATHPOLY *poly, int index);
 static void allowCoeffActions (BOOL active);
 static int getSelectedCoeff (GtkTreeSelection *selection);
 static void treeSelectionCallback (GtkTreeSelection *treeselection, gpointer user_data);
 
-static GtkTreeView* createCoeffListTreeView (GtkTreeView **pOther);
+static GtkTreeView* coeffCreateListTreeView (GtkTreeView **pOther);
 static void indexCellDataFunc (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
                                GtkTreeModel *model, GtkTreeIter *iter, gpointer data);
 static void coeffCellDataFunc (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
                                GtkTreeModel *model, GtkTreeIter *iter, gpointer data);
-static void fillCoeffListTreeView (GtkTreeView *tree,  GtkListStore *store,
+static void coeffFillListTreeView (GtkTreeView *tree,  GtkListStore *store,
                                    MATHPOLY *poly);
 static MATHPOLY *getSelectedPoly (FLTCOEFF* pFilter, int *pIndex);
 
@@ -268,7 +263,6 @@ static MATHPOLY *getSelectedPoly (FLTCOEFF* pFilter, int *pIndex)
 /** Operation which is performed when the \e Edit button is clicked (called from
  *  function mainDlgCoeffAction(), type is MAINDLG_COEFF_OPERATION).
  *
- *  \param widget       Widget that has emitted the "activate" signal.
  *  \param pFilter      Pointer to filter which is currently displayed in the
  *                      GtkTreeView coefficients list.
  *  \param poly         Pointer to selected coefficients (\p pFilter->num or
@@ -278,8 +272,7 @@ static MATHPOLY *getSelectedPoly (FLTCOEFF* pFilter, int *pIndex)
  *  \return             TRUE if the operation was really performed, else
  *                      FALSE (means no coefficient has changed).
  ******************************************************************************/
-static BOOL mainDlgCoeffEdit (GtkWidget *widget, FLTCOEFF *pFilter,
-                              MATHPOLY *poly, int index)
+static BOOL mainDlgCoeffEdit (FLTCOEFF *pFilter, MATHPOLY *poly, int index)
 {
     char *intro =
         g_strdup_printf (_("Changes coefficient of tap z<sup>-%d</sup> in"
@@ -298,7 +291,6 @@ static BOOL mainDlgCoeffEdit (GtkWidget *widget, FLTCOEFF *pFilter,
 /** Operation which is performed when the \e Factor button is clicked (called
  *  from function mainDlgCoeffAction(), type is MAINDLG_COEFF_OPERATION).
  *
- *  \param widget       Widget that has emitted the "activate" signal.
  *  \param pFilter      Pointer to filter which is currently displayed in the
  *                      GtkTreeView coefficients list.
  *  \param poly         Pointer to selected coefficients (\p pFilter->num or
@@ -308,8 +300,7 @@ static BOOL mainDlgCoeffEdit (GtkWidget *widget, FLTCOEFF *pFilter,
  *  \return             TRUE if the operation was really performed, else
  *                      FALSE (means no coefficient has changed).
  ******************************************************************************/
-static BOOL mainDlgCoeffsFactor (GtkWidget *widget, FLTCOEFF *pFilter,
-                                 MATHPOLY *poly, int index)
+static BOOL mainDlgCoeffsFactor (FLTCOEFF *pFilter, MATHPOLY *poly, int index)
 {
     int i;
     double factor = 1.0;
@@ -336,13 +327,6 @@ static BOOL mainDlgCoeffsFactor (GtkWidget *widget, FLTCOEFF *pFilter,
 /** Operation which is performed when the \e Round button is clicked (called
  *  from function mainDlgCoeffAction(), type is MAINDLG_COEFF_OPERATION).
  *
- *  \attention          If the "activate" signal comes from a \c GtkButton
- *                      then the reference in \p widget cannot be passed to
- *                      gtk_message_dialog_new(), because this reference must
- *                      be of type GTK_WINDOW (but GTK_BUTTON is not inherited
- *                      from GTK_WINDOW).
- *
- *  \param widget       Widget that has emitted the "activate" signal.
  *  \param pFilter      Pointer to filter which is currently displayed in the
  *                      GtkTreeView coefficients list.
  *  \param poly         Pointer to selected coefficients (\p pFilter->num or
@@ -352,8 +336,7 @@ static BOOL mainDlgCoeffsFactor (GtkWidget *widget, FLTCOEFF *pFilter,
  *  \return             TRUE if the operation was really performed, else
  *                      FALSE (means no coefficient has changed).
  ******************************************************************************/
-static BOOL mainDlgCoeffsRound (GtkWidget *widget, FLTCOEFF *pFilter,
-                                MATHPOLY *poly, int index)
+static BOOL mainDlgCoeffsRound (FLTCOEFF *pFilter, MATHPOLY *poly, int index)
 {
     int i;
 
@@ -387,7 +370,7 @@ static BOOL mainDlgCoeffsRound (GtkWidget *widget, FLTCOEFF *pFilter,
 /** This function is called if a coefficients button (\e Edit, \e Factor,
  *  \e Round) or the associated menu item was clicked
  *
- *  \param widget       \c GtkButton that was clicked.
+ *  \param widget       Associated \c GtkButton or \c GtkTreeView widget.
  *  \param user_data    User data as passed to function g_signal_connect. Here
  *                      it is a pointer to the button descriptor in array
  *                      mainDlgCoeffBtn.
@@ -403,13 +386,13 @@ static void mainDlgCoeffAction (GtkWidget *widget, gpointer user_data)
     FLTCOEFF *pFilter = dfcPrjGetFilter ();
     MATHPOLY *poly = getSelectedPoly (&tmp, &index);
 
-    if ((pFilter != NULL) && (poly != NULL))                 /* sanity checks */
+    if ((pFilter != NULL) && (poly != NULL))               /* sanity checks */
     {
-        if (filterDuplicate (&tmp, pFilter) == 0)        /* make temp. filter */
+        if (filterDuplicate (&tmp, pFilter) == 0)      /* make temp. filter */
         {
-            if (pAction->op (widget, &tmp, poly, index))       /* performed? */
+            if (pAction->op (&tmp, poly, index))              /* performed? */
             {
-                result = filterCheck (&tmp);             /* check realization */
+                result = filterCheck (&tmp);           /* check realization */
 
                 if (FLTERR_CRITICAL (result))
                 {
@@ -419,7 +402,7 @@ static void mainDlgCoeffAction (GtkWidget *widget, gpointer user_data)
                                 " Maybe the result of such an operation"
                                 " leads to vanishing coefficients at all."));
                 } /* if */
-                else                   /* seems okay (do not free any memory) */
+                else                 /* seems okay (do not free any memory) */
                 {
                     dfcPrjSetFilter (FLTCLASS_NOTDEF, &tmp, NULL);
                     mainDlgUpdateFilter (result);
@@ -466,7 +449,6 @@ static void allowCoeffActions (BOOL active)
  *  \param user_data    Selection of the other GtkTreeView list (user data as
  *                      passed to function g_signal_connect).
  *
- *  \todo  Process double-clicks (see the tree-view tutorial on how to make this)
  ******************************************************************************/
 static void treeSelectionCallback (GtkTreeSelection *selection, gpointer user_data)
 {
@@ -490,6 +472,26 @@ static void treeSelectionCallback (GtkTreeSelection *selection, gpointer user_da
 } /* treeSelectionCallback() */
 
 
+/* FUNCTION *******************************************************************/
+/** Callback function for \e row-activated signal on a GtkTreeSelection associated
+ *  with a coefficients list GtkTreeView.
+ *
+ *  \param treeview     Selection of the GtkTreeView list.
+ *  \param path         Selection path.
+ *  \param column       A GtkTreeColumn.
+ *  \param user_data    Selection of the other GtkTreeView list (user data as
+ *                      passed to function g_signal_connect).
+ *
+ ******************************************************************************/
+static void treeDblClickedCallback (GtkTreeView *treeview, GtkTreePath *path,
+                                    GtkTreeViewColumn *column, gpointer userdata)
+{
+    (void) column;                                                /* unused */
+    (void) path;
+
+    mainDlgCoeffAction (GTK_WIDGET(treeview), userdata);
+}
+
 
 /* FUNCTION *******************************************************************/
 /** This function sets the \e markup property of an index cell instead of just
@@ -552,7 +554,7 @@ void coeffCellDataFunc (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
  *  \return         GtkTreeView widget associated with the coefficients list.
  *
  ******************************************************************************/
-static GtkTreeView* createCoeffListTreeView (GtkTreeView **pOther)
+static GtkTreeView* coeffCreateListTreeView (GtkTreeView **pOther)
 {
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
@@ -562,7 +564,6 @@ static GtkTreeView* createCoeffListTreeView (GtkTreeView **pOther)
      */
     GtkListStore *store = gtk_list_store_new (MAINDLG_LIST_COLUMN_SIZE,
                                               G_TYPE_INT, G_TYPE_DOUBLE);
-
     /* 2nd step: create the tree view
      */
     tree = GTK_TREE_VIEW (gtk_tree_view_new ());
@@ -571,7 +572,6 @@ static GtkTreeView* createCoeffListTreeView (GtkTreeView **pOther)
     gtk_tree_selection_set_mode (gtk_tree_view_get_selection (tree), GTK_SELECTION_SINGLE);
     gtk_tree_view_set_search_column (tree, MAINDLG_LIST_COLUMN_INDEX);
 
-
     /* 3rd step: create all columns
      */
     renderer = gtk_cell_renderer_text_new ();
@@ -596,20 +596,21 @@ static GtkTreeView* createCoeffListTreeView (GtkTreeView **pOther)
                                      GTK_TREE_VIEW_COLUMN_AUTOSIZE);
     gtk_tree_view_append_column (tree, column);
 
-
     /* 4th step: set model into tree view (and show widget)
      */
     gtk_tree_view_set_model (tree, GTK_TREE_MODEL (store));
     g_object_unref (G_OBJECT (store));        /* view now holds the reference */
 
-
-    /* 5th step: Connect to "changed" signal
+    /* 5th step: Connect to "changed" and "row-activated" signal
      */
     g_signal_connect_after ((gpointer) gtk_tree_view_get_selection (tree), "changed",
                             G_CALLBACK (treeSelectionCallback), pOther);
 
+    g_signal_connect(tree, "row-activated",
+                     G_CALLBACK (treeDblClickedCallback), &mainDlgCoeffBtn[0]);
+
     return tree;
-} /* createCoeffListTreeView() */
+} /* coeffCreateListTreeView() */
 
 
 
@@ -621,7 +622,7 @@ static GtkTreeView* createCoeffListTreeView (GtkTreeView **pOther)
  *  \param poly         Pointer to polynomial coefficients to be filled in.
  *
  ******************************************************************************/
-static void fillCoeffListTreeView (GtkTreeView *tree, GtkListStore *store,
+static void coeffFillListTreeView (GtkTreeView *tree, GtkListStore *store,
                                    MATHPOLY *poly)
 {
     int i;
@@ -633,23 +634,41 @@ static void fillCoeffListTreeView (GtkTreeView *tree, GtkListStore *store,
         gtk_list_store_set (store, &iter, MAINDLG_LIST_COLUMN_INDEX, i,
                             MAINDLG_LIST_COLUMN_COEFF, poly->coeff[i], -1);
     } /* for */
-} /* fillCoeffListTreeView() */
+} /* coeffFillListTreeView() */
 
 
 
 
+static void coeffRedrawListTreeViews ()
+{
+    GtkListStore *storeNum = GTK_LIST_STORE (gtk_tree_view_get_model (treeNumerator));
+    GtkListStore *storeDen = GTK_LIST_STORE (gtk_tree_view_get_model (treeDenominator));
+    FLTCOEFF* pFilter = dfcPrjGetFilter();
+
+    gtk_list_store_clear (storeNum);
+    gtk_list_store_clear (storeDen);
+
+    if (pFilter != NULL)
+    {
+        coeffFillListTreeView (treeNumerator, storeNum, &pFilter->num);
+        coeffFillListTreeView (treeDenominator, storeDen, &pFilter->den);
+    } /* if */
+
+} /* coeffRedrawListTreeViews() */
+
+
+
 /* EXPORTED FUNCTION DEFINITIONS **********************************************/
 
 
 /* FUNCTION *******************************************************************/
-/** \e dfcgen top widget creation.
+/** DFCGen top widget creation.
  *
  *  This function is completely taken over from interface.c, where it has been
  *  generated by \e glade.
  *
  *
  *  \return     Top widget reference.
- *  \todo       Reduce number of variables used (optimize the function)
  ******************************************************************************/
 GtkWidget* mainDlgCreate (void)
 {
@@ -677,7 +696,7 @@ GtkWidget* mainDlgCreate (void)
         if (iconPixbuf != NULL)
         {
             gtk_window_set_icon (GTK_WINDOW (topWidget), iconPixbuf);
-            gdk_pixbuf_unref (iconPixbuf);
+            g_object_unref (iconPixbuf);
         } /* if */
     }
 #endif /* G_OS_WIN32 */
@@ -982,7 +1001,7 @@ GtkWidget* mainDlgCreate (void)
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (widget), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
 
-    treeDenominator = createCoeffListTreeView (&treeNumerator);
+    treeDenominator = coeffCreateListTreeView (&treeNumerator);
     gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (treeDenominator));
     gtk_widget_set_tooltip_text (GTK_WIDGET (treeDenominator), _("Denominator coefficients"));
 
@@ -993,7 +1012,7 @@ GtkWidget* mainDlgCreate (void)
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (widget), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
 
-    treeNumerator = createCoeffListTreeView (&treeDenominator);
+    treeNumerator = coeffCreateListTreeView (&treeDenominator);
     gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (treeNumerator));
     gtk_widget_set_tooltip_text (GTK_WIDGET (treeNumerator), _("Numerator coefficients"));
 
@@ -1133,8 +1152,6 @@ void mainDlgUpdatePrjInfo ()
  ******************************************************************************/
 BOOL mainDlgUpdateFilter (int err)
 {
-    GtkListStore *storeNum, *storeDen;
-
     FLTCOEFF* pFilter = dfcPrjGetFilter();
     BOOL valid = pFilter != NULL;
 
@@ -1161,19 +1178,10 @@ BOOL mainDlgUpdateFilter (int err)
         gtk_widget_set_sensitive (lookup_widget (topWidget, "menuItemFileExport"), valid);
         gtk_widget_set_sensitive (lookup_widget (topWidget, "menuItemFilePrint"), valid);
 
-        storeNum = GTK_LIST_STORE (gtk_tree_view_get_model (treeNumerator));
-        storeDen = GTK_LIST_STORE (gtk_tree_view_get_model (treeDenominator));
-        gtk_list_store_clear (storeNum);
-        gtk_list_store_clear (storeDen);
-
-        if (valid)
-        {
-            fillCoeffListTreeView (treeNumerator, storeNum, &pFilter->num);
-            fillCoeffListTreeView (treeDenominator, storeDen, &pFilter->den);
-        } /* if */
-
+        coeffRedrawListTreeViews ();
         responseWinRedraw (RESPONSE_TYPE_SIZE);
         rootsPlotUpdate (pFilter);
+
         return TRUE;
     } /* if */
 
@@ -1182,6 +1190,19 @@ BOOL mainDlgUpdateFilter (int err)
 
 
 
+/* FUNCTION *******************************************************************/
+/** Redraw of main filter dialog and all associated response plot (an case a
+ *  project is defined, else does nothing).
+ *
+ ******************************************************************************/
+void mainDlgRedrawAll ()
+{
+    responseWinRedraw (RESPONSE_TYPE_SIZE);     /* redraw all response plots */
+    rootsPlotRedraw ();                /* redraw roots window in main dialog */
+    coeffRedrawListTreeViews (); /* redraw coefficients lists in main dialog */
+}
+
+
 
 /* FUNCTION *******************************************************************/
 /** Adjustment of main filter dialog from a new project (may be read from file
