@@ -3,16 +3,15 @@
  * \file     projectFile.c
  * \brief    Project file handling.
  *
- * \note     The \e dfcgen filter project file functions implemented here are
+ * \note     The DFCGen filter project file functions implemented here are
  *           very closely related to the \e GLib XML support functions.
  *           Adoption to other platforms or to libxml is possible, but need
  *           some rework.
  *
- * \note     All double values written to a \e dfcgen project file are formated
+ * \note     All double values written to a DFCGen project file are formated
  *           in the \e C locale for LC_CTYPE.
  *
- * \author   Copyright (C) 2006, 2011-2012 Ralf Hoppe <ralf.hoppe@ieee.org>
- * \version  $Id: projectFile.c 234 2012-06-08 15:32:33Z ralf $
+ * \author   Copyright (C) 2006, 2011-2012 Ralf Hoppe <ralf.hoppe@dfcgen.de>
  *
  ******************************************************************************/
 
@@ -45,7 +44,7 @@ typedef enum
     PRJF_TAGFLAG_MISC = 1 << FLTCLASS_MISC, /**< Tag is mandatory for misc filters */
     PRJF_TAGFLAG_LINFIR = 1 << FLTCLASS_LINFIR, /**< Tag is mandatory for linear FIR filters */
     PRJF_TAGFLAG_STDIIR = 1 << FLTCLASS_STDIIR, /**< Tag is mandatory for standard IIR filters */
-    PRJF_TAGFLAG_FOUND = 1 << FLTCLASS_SIZE /**< Tag found (in \e dfcgen project file) */
+    PRJF_TAGFLAG_FOUND = 1 << FLTCLASS_SIZE /**< Tag found (in DFCGen project file) */
 } PRJF_TAGFLAG;
 
 
@@ -83,8 +82,6 @@ typedef int (*PRJF_TAG_OPEN_HANDLER) (GMarkupParseContext *ctx,
 
 
 /** Project file tag identifiers.
- *
- *  \attention Do not change because used as index.
  */
 typedef enum
 {
@@ -138,8 +135,6 @@ struct _PRJF_TAG_DESC
 
 
 /** Keyword identifier in an export template file.
- *
- *  \attention Do not change because used as index.
  */
 typedef enum
 {
@@ -313,7 +308,7 @@ static PRJF_TAG_DESC prjTagsDesc[PRJF_IDTAG_SIZE] =
  *
  *  \param ctx          Pointer to parser context.
  *  \param pTag         Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param text         Pointer to content of tag (zero-terminated).
  *
  *  \return             Zero on success, else an error number.
@@ -341,7 +336,7 @@ static int tagStringHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *pTag,
  *
  *  \param ctx          Pointer to parser context.
  *  \param p            Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param text         Pointer to content of tag (zero-terminated).
  *
  *  \return             Zero on success, else an error number.
@@ -370,7 +365,7 @@ static int tagIntHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *p,
  *
  *  \param ctx          Pointer to parser context.
  *  \param p            Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param text         Pointer to content of tag (zero-terminated).
  *
  *  \return             Zero on success, else an error number.
@@ -403,7 +398,7 @@ static int tagDoubleHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *p,
  *
  *  \param ctx          Pointer to parser context.
  *  \param pTag         Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param text         Pointer to content of tag (zero-terminated).
  *
  *  \return             Zero on success, else an error number in domain G_MARKUP_ERROR.
@@ -439,7 +434,7 @@ static int tagCoeffHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *pTag,
  *
  *  \param ctx          Pointer to parser context.
  *  \param pTag         Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param text         Pointer to content of tag (zero-terminated).
  *
  *  \return             Zero on success, else an error number.
@@ -481,7 +476,7 @@ static int tagDegreeHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *pTag,
  *
  *  \param ctx          Pointer to parser context.
  *  \param p            Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param attrn        List of attribute names.
  *  \param attrv        List of associated attribute values.
  *
@@ -502,7 +497,7 @@ static int tagPolyHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *p,
  *
  *  \param ctx          Pointer to parser context.
  *  \param pTag         Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param attrn        List of attribute names.
  *  \param attrv        List of associated attribute values.
  *
@@ -539,7 +534,7 @@ static int attrSupersedHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *pTag,
  *
  *  \param ctx          Pointer to parser context.
  *  \param pTag         Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param attrn        List of attribute names.
  *  \param attrv        List of associated attribute values.
  *
@@ -594,7 +589,7 @@ static int attrStringHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *pTag,
  *
  *  \param ctx          Pointer to parser context.
  *  \param pTag         Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param attrn        List of attribute names.
  *  \param attrv        List of associated attribute values.
  *
@@ -630,7 +625,7 @@ static int attrGeometricHandler (GMarkupParseContext *ctx, PRJF_TAG_DESC *pTag,
  *
  *  \param ctx          Pointer to parser context.
  *  \param pTag         Pointer to tag descriptor.
- *  \param prj          Pointer to \e dfcgen project.
+ *  \param prj          Pointer to DFCGen project.
  *  \param attrn        List of attribute names.
  *  \param attrv        List of associated attribute values.
  *
@@ -1064,11 +1059,14 @@ static int writeProject (FILE *f, DFCPRJ_FILTER *prj)
 
 
 /* FUNCTION *******************************************************************/
-/** Reads a filter project from a file into variable tmpPrj. In case of an error
- *  it sets the pointer to the error structure, which itself can be used to
- *  display the original error message. Modification of the error structure
- *  pointer indicates an critical error. In that case it has to be free'ed with
- *  g_error_free().
+/**
+ *  \brief              Reads a filter project from a file into variable tmpPrj.
+ *
+ *                      In case of an error it sets the pointer to the error
+ *                      structure, which itself can be used to display the
+ *                      original error message. Modification of the error
+ *                      structure pointer indicates an critical error. In that
+ *                      case it has to be free'ed with g_error_free().
  *
  *  \param filename     Filename of file (with path).
  *  \param flags        Project flags to be preset. Set this to
@@ -1108,6 +1106,7 @@ static void readProject (const char *filename, unsigned flags, GError **err)
 
 
     ctx = g_markup_parse_context_new (&parser, 0, &tmpPrj, NULL);
+    ASSERT (ctx != NULL);
 
     if (g_file_get_contents (filename, &content, &len, err))     /* read file */
     {
@@ -1151,6 +1150,30 @@ static void readProject (const char *filename, unsigned flags, GError **err)
                         } /* if */
                     } /* if */
                 } /* for */
+
+                /*
+                 * The next step is a counterpart to writeFrequTransf().  It
+                 * restores the cutoff/center frequency the right way, when
+                 * frequency transformations are active.
+                 */
+                switch (tmpPrj.design.all.ftr.type)
+                {
+                    case FTR_NON:                  /* ftr.fc has no meaning */
+                    case FTR_BANDPASS: /* wrote ftr.fc with PRJF_TAG_CENTER */
+                    case FTR_BANDSTOP:
+                        break;
+
+                    case FTR_HIGHPASS: /* wrote ftr.fc with PRJF_TAG_CUTOFF */
+                        tmpPrj.design.all.ftr.fc = tmpPrj.design.all.cutoff;
+                        break;
+
+                    default:
+                        g_set_error (err, G_MARKUP_ERROR,
+                                     G_MARKUP_ERROR_INVALID_CONTENT,
+                                     _("Unknown frequency transform type %d"),
+                                     tmpPrj.design.all.ftr.type);
+                        break;
+                } /* switch */
             } /* if */
         } /* if */
 
@@ -1529,7 +1552,7 @@ void prjFileRead (const char *filename, DFCPRJ_FILTER *pProject, GError **err)
 
 
 /* FUNCTION *******************************************************************/
-/** Scans for project info (header) in a \e dfcgen project file. In case
+/** Scans for project info (header) in a DFCGen project file. In case
  *  of an error it sets the pointer to the error structure, which itself can be
  *  used to display the original error message. Modification of the error
  *  structure pointer indicates an critical error. In that case it has to be
