@@ -3,8 +3,7 @@
  * \file     responseDlg.c
  * \brief    Response settings/properties dialog.
  *
- * \author   Copyright (C) 2006, 2011-2012 Ralf Hoppe <ralf.hoppe@ieee.org>
- * \version  $Id: responseDlg.c 240 2012-06-08 15:52:14Z ralf $
+ * \author   Copyright (C) 2006, 2011-2012, 2020 Ralf Hoppe <ralf.hoppe@dfcgen.de>
  *
  ******************************************************************************/
 
@@ -372,9 +371,17 @@ GtkWidget* responseDlgCreate (PLOT_DIAG *pDiag)
     gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (widget), TRUE);
     GLADE_HOOKUP_OBJECT (responseDlg, widget, RESPONSE_DLG_SPIN_SAMPLES);
     gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), pDiag->num);
     gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
 
+    if (pDiag->initFunc == NULL)
+    {
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), pDiag->num);
+    } /* if */
+    else
+    {
+        gtk_widget_set_sensitive (widget, FALSE);
+    } /* else */
+
     label = gtk_label_new (NULL);                             /* x-start unit */
     gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
                       (GtkAttachOptions) (GTK_FILL),
