@@ -3,8 +3,7 @@
  * \file
  *           Digital filter response plotter.
  *
- * \author   Copyright (C) 2006, 2011 Ralf Hoppe <ralf.hoppe@ieee.org>
- * \version  $Id: responsePlot.c 105 2011-08-21 15:23:11Z ralf $
+ * \author   Copyright (C) 2006, 2011, 2020 Ralf Hoppe <ralf.hoppe@dfcgen.de>
  *
  ******************************************************************************/
 
@@ -66,41 +65,49 @@ static void timeResponseEnd (void *pData);
  */
 static RESPONSE_PLOT responsePlot[RESPONSE_TYPE_SIZE] =
 {
+    [RESPONSE_TYPE_MAGNITUDE] =
     {
-        RESPONSE_TYPE_MAGNITUDE,                                      /* type */
-        plotMagnitude,                                          /* sampleFunc */
+        .type = RESPONSE_TYPE_MAGNITUDE,
+        .sampleFunc = plotMagnitude,
     },
+    [RESPONSE_TYPE_ATTENUATION] =
     {
-        RESPONSE_TYPE_ATTENUATION,
-        plotAttenuation,
+        .type = RESPONSE_TYPE_ATTENUATION,
+        .sampleFunc = plotAttenuation,
     },
+    [RESPONSE_TYPE_CHAR] =
     {
-        RESPONSE_TYPE_CHAR,
-        plotChar,
+        .type = RESPONSE_TYPE_CHAR,
+        .sampleFunc = plotChar,
     },
+    [RESPONSE_TYPE_PHASE] =
     {
-        RESPONSE_TYPE_PHASE,
-        plotPhase,
+        .type = RESPONSE_TYPE_PHASE,
+        .sampleFunc = plotPhase,
     },
+    [RESPONSE_TYPE_DELAY] =
     {
-        RESPONSE_TYPE_DELAY,
-        plotPhaseDelay,
+        .type = RESPONSE_TYPE_DELAY,
+        .sampleFunc = plotPhaseDelay,
     },
+    [RESPONSE_TYPE_GROUP] =
     {
-        RESPONSE_TYPE_GROUP,
-        plotGroupDelay,
+        .type = RESPONSE_TYPE_GROUP,
+        .sampleFunc = plotGroupDelay,
     },
+    [RESPONSE_TYPE_IMPULSE] =
     {
-        RESPONSE_TYPE_IMPULSE,
-        timeResponse,                                           /* sampleFunc */
-        plotImpulseInit,                                          /* initFunc */
-        timeResponseEnd,                                           /* endFunc */
+        .type = RESPONSE_TYPE_IMPULSE,
+        .sampleFunc = timeResponse,
+        .initFunc = plotImpulseInit,
+        .endFunc = timeResponseEnd,
     },
+    [RESPONSE_TYPE_STEP] =
     {
-        RESPONSE_TYPE_STEP,
-        timeResponse,                                           /* sampleFunc */
-        plotStepInit,                                             /* initFunc */
-        timeResponseEnd,                                           /* endFunc */
+        .type = RESPONSE_TYPE_STEP,
+        .sampleFunc = timeResponse,
+        .initFunc = plotStepInit,
+        .endFunc = timeResponseEnd
     }
 }; /* responsePlot[] */
 
