@@ -3,8 +3,7 @@
  * \file
  *           Digital filter response window creation and callbacks.
  *
- * \author   Copyright (C) 2006, 2011, 2012 Ralf Hoppe <ralf.hoppe@ieee.org>
- * \version  $Id: responseWin.c 254 2012-06-10 16:32:45Z ralf $
+ * \author   Copyright (C) 2006, 2011, 2012, 2020 Ralf Hoppe <ralf.hoppe@dfcgen.de>
  *
  ******************************************************************************/
 
@@ -41,7 +40,7 @@ typedef struct
     GdkGCValues original;                /**< Saved GdkGC values when zooming */
     GtkCheckMenuItem *menuref;            /**< (Backward) menu item reference */
     GtkWidget* btnPrint;                  /**< Print button widget reference */
-    GtkWidget* topWidget;        /**< response plot window (top-level widget) */
+    GtkWidget* topWidget; /**< response plot top-level widget (NULL if not exists) */
     GtkWidget* draw;                            /**< \e GtkDrawingArea widget */
     GtkWidget* label;             /**< Label which shows the number of points */
 } RESPONSE_WIN;
@@ -50,7 +49,7 @@ typedef struct
 /* LOCAL CONSTANT DEFINITIONS *************************************************/
 
 
-#define RESPONSE_WIN_GRAPH_THICKNESS    (1.5)           /**< Graph line width */
+#define RESPONSE_WIN_GRAPH_THICKNESS    (2.0)           /**< Graph line width */
 
 
 /** Used GDK graphics context for zoom rectangle
@@ -65,8 +64,8 @@ static void cancelZoomMode (RESPONSE_WIN *pDesc);
 static void responseWinCreate (RESPONSE_WIN *pDesc);
 static gboolean exposeHandler (GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
 static BOOL responseWinExpose (RESPONSE_WIN* pDesc);
-static void responseWinDrawAreaMapped (GtkWidget* widget, gpointer user_data);
-static void responseWinDrawAreaDestroy (GtkObject* object, gpointer user_data);
+static void responseWinMapped (GtkWidget* widget, gpointer user_data);
+static void responseWinDestroyed (GtkObject* object, gpointer user_data);
 static void responseWinBtnPropActivate (GtkButton *button, gpointer user_data);
 static void responseWinBtnPrintActivate (GtkWidget* srcWidget, gpointer user_data);
 static gboolean responseWinKeyPress (GtkWidget *widget, GdkEventKey *event,
@@ -87,6 +86,7 @@ static PLOT_UNIT plotUnitDeg = {"°", 1.0};          /**< Constant degree unit *
  */
 static RESPONSE_WIN responseWidget[RESPONSE_TYPE_SIZE] =
 {
+    [RESPONSE_TYPE_MAGNITUDE] =
     {
         RESPONSE_TYPE_MAGNITUDE,                                      /* type */
         "amplitude.png",                                          /* iconFile */
@@ -97,6 +97,7 @@ static RESPONSE_WIN responseWidget[RESPONSE_TYPE_SIZE] =
             .thickness = RESPONSE_WIN_GRAPH_THICKNESS,    /* graph thickness */
         },
     },
+    [RESPONSE_TYPE_ATTENUATION] =
     {
         RESPONSE_TYPE_ATTENUATION,
         "attenuation.png",                                        /* iconFile */
@@ -107,6 +108,7 @@ static RESPONSE_WIN responseWidget[RESPONSE_TYPE_SIZE] =
             .thickness = RESPONSE_WIN_GRAPH_THICKNESS,    /* graph thickness */
         },
     },
+    [RESPONSE_TYPE_CHAR] =
     {
         RESPONSE_TYPE_CHAR,
         "charfunc.png",                                           /* iconFile */
@@ -117,6 +119,7 @@ static RESPONSE_WIN responseWidget[RESPONSE_TYPE_SIZE] =
             .thickness = RESPONSE_WIN_GRAPH_THICKNESS,    /* graph thickness */
         },
     },
+    [RESPONSE_TYPE_PHASE] =
     {
         RESPONSE_TYPE_PHASE,
         "phase.png",                                              /* iconFile */
@@ -127,6 +130,7 @@ static RESPONSE_WIN responseWidget[RESPONSE_TYPE_SIZE] =
             .thickness = RESPONSE_WIN_GRAPH_THICKNESS,    /* graph thickness */
         },
     },
+    [RESPONSE_TYPE_DELAY] =
     {
         RESPONSE_TYPE_DELAY,
         "phasedelay.png",                                         /* iconFile */
@@ -137,6 +141,7 @@ static RESPONSE_WIN responseWidget[RESPONSE_TYPE_SIZE] =
             .thickness = RESPONSE_WIN_GRAPH_THICKNESS,    /* graph thickness */
         }
     },
+    [RESPONSE_TYPE_GROUP] =
     {
         RESPONSE_TYPE_GROUP,
         "grpdelay.png",                                           /* iconFile */
@@ -147,6 +152,7 @@ static RESPONSE_WIN responseWidget[RESPONSE_TYPE_SIZE] =
             .thickness = RESPONSE_WIN_GRAPH_THICKNESS,    /* graph thickness */
         }
     },
+    [RESPONSE_TYPE_IMPULSE] =
     {
         RESPONSE_TYPE_IMPULSE,
         "impulse.png",                                            /* iconFile */
@@ -157,6 +163,7 @@ static RESPONSE_WIN responseWidget[RESPONSE_TYPE_SIZE] =
             .thickness = RESPONSE_WIN_GRAPH_THICKNESS,    /* graph thickness */
         },
     },
+    [RESPONSE_TYPE_STEP] =
     {
         RESPONSE_TYPE_STEP,
         "step.png",                                               /* iconFile */
@@ -273,9 +280,6 @@ static void responseWinCreate (RESPONSE_WIN *pDesc)
 
     pDesc->topWidget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
-    gtk_window_set_skip_taskbar_hint (GTK_WINDOW (pDesc->topWidget), TRUE); /* added */
-    gtk_window_set_skip_pager_hint (GTK_WINDOW (pDesc->topWidget), TRUE); /* added */
-
     gtk_window_set_title (GTK_WINDOW (pDesc->topWidget),
                           gtk_label_get_text (
                               GTK_LABEL (
@@ -289,7 +293,7 @@ static void responseWinCreate (RESPONSE_WIN *pDesc)
     if (iconPixbuf != NULL)
     {
         gtk_window_set_icon (GTK_WINDOW (pDesc->topWidget), iconPixbuf);
-        gdk_pixbuf_unref (iconPixbuf);
+        g_object_unref (iconPixbuf);
     } /* if */
 
     vbox = gtk_vbox_new (FALSE, 0);
@@ -344,10 +348,10 @@ static void responseWinCreate (RESPONSE_WIN *pDesc)
                       G_CALLBACK (exposeHandler),
                       pDesc);
     g_signal_connect ((gpointer) pDesc->draw, "destroy",
-                      G_CALLBACK (responseWinDrawAreaDestroy),
+                      G_CALLBACK (responseWinDestroyed),
                       pDesc);
     g_signal_connect ((gpointer) pDesc->draw, "map",
-                      G_CALLBACK (responseWinDrawAreaMapped),
+                      G_CALLBACK (responseWinMapped),
                       pDesc);
     g_signal_connect ((gpointer) btnSettings, "clicked",
                       G_CALLBACK (responseWinBtnPropActivate),
@@ -450,7 +454,7 @@ static gboolean exposeHandler (GtkWidget *widget, GdkEventExpose *event, gpointe
  *                      event.
  *
  ******************************************************************************/
-static void responseWinDrawAreaDestroy(GtkObject* object, gpointer user_data)
+static void responseWinDestroyed(GtkObject* object, gpointer user_data)
 {
     RESPONSE_WIN* pDesc = user_data;
 
@@ -461,7 +465,7 @@ static void responseWinDrawAreaDestroy(GtkObject* object, gpointer user_data)
                               PLOT_COLOR_SIZE);
     gtk_check_menu_item_set_active (pDesc->menuref, FALSE);    /* update menu */
 
-} /* responseWinDrawAreaDestroy() */
+} /* responseWinDestroyed() */
 
 
 
@@ -474,7 +478,7 @@ static void responseWinDrawAreaDestroy(GtkObject* object, gpointer user_data)
  *                      event.
  *
  ******************************************************************************/
-static void responseWinDrawAreaMapped (GtkWidget* widget, gpointer user_data)
+static void responseWinMapped (GtkWidget* widget, gpointer user_data)
 {
     gboolean success[PLOT_COLOR_SIZE];
 
@@ -490,7 +494,7 @@ static void responseWinDrawAreaMapped (GtkWidget* widget, gpointer user_data)
         DEBUG_LOG ("Could not allocate colors in system colormap");
     } /* if */
 
-} /* responseWinDrawAreaMapped() */
+} /* responseWinMapped() */
 
 
 
@@ -608,7 +612,11 @@ void responseWinRedraw (RESPONSE_TYPE type)
 
     for (type = start; type <= stop; type++)
     {
-        gtk_widget_set_sensitive (responseWidget[type].btnPrint, filterValid);
+        if (responseWidget[type].topWidget != NULL)
+        {
+            gtk_widget_set_sensitive (responseWidget[type].btnPrint, filterValid);
+        } /* if */
+
         (void)responseWinExpose (&responseWidget[type]);
     } /* for */
 } /* responseWinRedraw() */
