@@ -3,8 +3,7 @@
  * \file
  *           Standard IIR filter dialog functions.
  *
- * \author   Copyright (C) 2006, 2011-2012 Ralf Hoppe
- * \version  $Id: stdIirDesignDlg.c 179 2012-05-19 15:56:13Z ralf $
+ * \author   Copyright (C) 2006, 2011-2012, 2020 Ralf Hoppe
  *
  ******************************************************************************/
 
@@ -88,12 +87,12 @@ static STDIIR_DLG_CHAR stdIirDlgChar[STDIIR_TYPE_SIZE] =
 }; /* stdIirDlgChar[] */
 
 
-static const gboolean ftrEntry[FTR_SIZE][4] =
-{ /* entryCutF, entryCenterF, entryBandwidth, checkGeometric */
-    {TRUE,      FALSE,        FALSE,          FALSE},          /* FTR_NON */
-    {FALSE,     TRUE,         FALSE,          FALSE},     /* FTR_HIGHPASS */
-    {FALSE,     TRUE,         TRUE,           TRUE},      /* FTR_BANDPASS */
-    {FALSE,     TRUE,         TRUE,           TRUE},      /* FTR_BANDSTOP */
+static const gboolean ftrEntry[FTR_SIZE][5] =
+{                  /* entryCutF, entryCenterF, entryBandwidth, checkGeometric, evenDegree */
+    [FTR_NON]      = {TRUE,      FALSE,        FALSE,          FALSE,          FALSE},
+    [FTR_HIGHPASS] = {FALSE,     TRUE,         FALSE,          FALSE,          FALSE},
+    [FTR_BANDPASS] = {FALSE,     TRUE,         TRUE,           TRUE,           TRUE},
+    [FTR_BANDSTOP] = {FALSE,     TRUE,         TRUE,           TRUE,           TRUE},
 };
 
 
@@ -780,6 +779,11 @@ int stdIirDesignDlgApply (GtkWidget *topWidget, const CFG_DESKTOP* pPrefs)
             design.ftr.type = idx;
         } /* else */
 
+        if (ftrEntry[design.ftr.type][4] && GSL_IS_ODD(design.order))
+        {
+            dlgError(topWidget, _("Degree must be even for bandpass/bandstop."));
+            return INT_MAX;
+        }
 
         if (ftrEntry[design.ftr.type][0]) /* lowpass cutoff frequency required? */
         {
